@@ -4,6 +4,7 @@ import (
 	"bufio"
 	"bytes"
 	"context"
+	"encoding/json"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -116,9 +117,14 @@ func (c colimaApp) Start(conf config.Config) error {
 		log.Error(fmt.Errorf("error persisting runtime settings: %w", err))
 	}
 
+	// persist the kubernetes config
+	if err := c.setKubernetes(conf.Kubernetes); err != nil {
+		log.Error(fmt.Errorf("error persisting kubernetes settings: %w", err))
+	}
+
 	log.Println("done")
 
-	if err := generateSSHConfig(); err != nil {
+	if err := generateSSHConfig(conf.SSHConfig); err != nil {
 		log.Trace("error generating ssh_config: %w", err)
 	}
 	return nil
@@ -162,7 +168,7 @@ func (c colimaApp) Stop(force bool) error {
 
 	log.Println("done")
 
-	if err := generateSSHConfig(); err != nil {
+	if err := generateSSHConfig(false); err != nil {
 		log.Trace("error generating ssh_config: %w", err)
 	}
 	return nil
@@ -209,7 +215,7 @@ func (c colimaApp) Delete() error {
 
 	log.Println("done")
 
-	if err := generateSSHConfig(); err != nil {
+	if err := generateSSHConfig(false); err != nil {
 		log.Trace("error generating ssh_config: %w", err)
 	}
 	return nil
@@ -306,12 +312,25 @@ func (c colimaApp) Status() error {
 		return err
 	}
 
-	log.Println(config.CurrentProfile().DisplayName, "is running")
+	driver := "QEMU"
+	conf, _ := limautil.InstanceConfig()
+	if !conf.Empty() {
+		driver = conf.DriverLabel()
+	}
+
+	log.Println(config.CurrentProfile().DisplayName, "is running using", driver)
 	log.Println("arch:", c.guest.Arch())
 	log.Println("runtime:", currentRuntime)
-	if conf, err := limautil.InstanceConfig(); err == nil {
+	if conf.MountType != "" {
 		log.Println("mountType:", conf.MountType)
 	}
+
+	// ip address
+	if ipAddress := limautil.IPAddress(config.CurrentProfile().ID); ipAddress != "127.0.0.1" {
+		log.Println("address:", ipAddress)
+	}
+
+	// docker socket
 	if currentRuntime == docker.Name {
 		log.Println("socket:", "unix://"+docker.HostSocketFile())
 	}
@@ -377,6 +396,15 @@ func (c colimaApp) setRuntime(runtime string) error {
 	return c.guest.Set(environment.ContainerRuntimeKey, runtime)
 }
 
+func (c colimaApp) setKubernetes(conf config.Kubernetes) error {
+	b, err := json.Marshal(conf)
+	if err != nil {
+		return err
+	}
+
+	return c.guest.Set(kubernetes.ConfigKey, string(b))
+}
+
 func (c colimaApp) currentContainerEnvironments(ctx context.Context) ([]environment.Container, error) {
 	var containers []environment.Container
 
@@ -430,7 +458,7 @@ func (c colimaApp) Active() bool {
 	return c.guest.Running(context.Background())
 }
 
-func generateSSHConfig() error {
+func generateSSHConfig(modifySSHConfig bool) error {
 	instances, err := limautil.Instances()
 	if err != nil {
 		return fmt.Errorf("error retrieving instances: %w", err)
@@ -463,6 +491,11 @@ func generateSSHConfig() error {
 		return fmt.Errorf("error writing ssh_config file: %w", err)
 	}
 
+	if !modifySSHConfig {
+		// ~/.ssh/config modification disabled
+		return nil
+	}
+
 	includeLine := "Include " + sshFileColima
 
 	sshFileSystem := filepath.Join(util.HomeDir(), ".ssh", "config")
