@@ -11,7 +11,6 @@ import (
 	"github.com/abiosoft/colima/cmd/root"
 	"github.com/abiosoft/colima/config"
 	"github.com/abiosoft/colima/config/configmanager"
-	"github.com/abiosoft/colima/daemon/process/gvproxy"
 	"github.com/abiosoft/colima/embedded"
 	"github.com/abiosoft/colima/environment"
 	"github.com/abiosoft/colima/environment/container/docker"
@@ -37,7 +36,9 @@ Run 'colima template' to set the default configurations or 'colima start --edit'
 		"  colima start --runtime containerd --kubernetes\n" +
 		"  colima start --cpu 4 --memory 8 --disk 100\n" +
 		"  colima start --arch aarch64\n" +
-		"  colima start --dns 1.1.1.1 --dns 8.8.8.8",
+		"  colima start --dns 1.1.1.1 --dns 8.8.8.8\n" +
+		"  colima start --dns-host example.com=1.2.3.4\n" +
+		"  colima start --kubernetes --kubernetes-disable=coredns,servicelb,traefik,local-storage,metrics-server",
 	Args: cobra.MaximumNArgs(1),
 	RunE: func(cmd *cobra.Command, args []string) error {
 		app := newApp()
@@ -93,10 +94,10 @@ const (
 	defaultMemory            = 2
 	defaultDisk              = 60
 	defaultKubernetesVersion = kubernetes.DefaultVersion
-	defaultMountType         = "sshfs"
-	defaultNetworkDriver     = gvproxy.Name
 )
 
+var defaultKubernetesDisable = []string{"traefik"}
+
 var startCmdArgs struct {
 	config.Config
 
@@ -106,13 +107,23 @@ var startCmdArgs struct {
 		Edit             bool
 		Editor           string
 		ActivateRuntime  bool
+		DNSHosts         []string
 	}
 }
 
 func init() {
 	runtimes := strings.Join(environment.ContainerRuntimes(), ", ")
-	networkDrivers := strings.Join([]string{"slirp", gvproxy.Name}, ", ")
-	defaultArch := string(environment.HostArch().Value())
+	defaultArch := string(environment.HostArch())
+
+	defaultMountType := "9p"
+	defaultVMType := "qemu"
+	if util.MacOS13OrNewer() {
+		defaultVMType = "vz"
+		defaultMountType = "virtiofs"
+	}
+
+	mounts := strings.Join([]string{defaultMountType, "sshfs"}, ", ")
+	types := strings.Join([]string{defaultVMType, "qemu"}, ", ")
 
 	root.Cmd().AddCommand(startCmd)
 	startCmd.Flags().StringVarP(&startCmdArgs.Runtime, "runtime", "r", docker.Name, "container runtime ("+runtimes+")")
@@ -125,9 +136,11 @@ func init() {
 
 	// network
 	if util.MacOS() {
-		startCmd.Flags().StringVar(&startCmdArgs.Network.Driver, "network-driver", defaultNetworkDriver, "network driver to use ("+networkDrivers+")")
 		startCmd.Flags().BoolVar(&startCmdArgs.Network.Address, "network-address", false, "assign reachable IP address to the VM")
 	}
+	if util.MacOS13OrNewer() {
+		startCmd.Flags().StringVarP(&startCmdArgs.VMType, "vm-type", "t", defaultVMType, "virtual machine type ("+types+")")
+	}
 
 	// config
 	startCmd.Flags().BoolVarP(&startCmdArgs.Flags.Edit, "edit", "e", false, "edit the configuration file before starting")
@@ -135,16 +148,19 @@ func init() {
 
 	// mounts
 	startCmd.Flags().StringSliceVarP(&startCmdArgs.Flags.Mounts, "mount", "V", nil, "directories to mount, suffix ':w' for writable")
-	startCmd.Flags().StringVar(&startCmdArgs.MountType, "mount-type", defaultMountType, "volume driver for the mount (sshfs, 9p)")
+	startCmd.Flags().StringVar(&startCmdArgs.MountType, "mount-type", defaultMountType, "volume driver for the mount ("+mounts+")")
 
 	// ssh agent
 	startCmd.Flags().BoolVarP(&startCmdArgs.ForwardAgent, "ssh-agent", "s", false, "forward SSH agent to the VM")
 
+	// ssh config generation
+	startCmd.Flags().BoolVar(&startCmdArgs.SSHConfig, "ssh-config", true, "generate SSH config in ~/.ssh/config")
+
 	// k8s
 	startCmd.Flags().BoolVarP(&startCmdArgs.Kubernetes.Enabled, "kubernetes", "k", false, "start with Kubernetes")
 	startCmd.Flags().BoolVar(&startCmdArgs.Flags.LegacyKubernetes, "with-kubernetes", false, "start with Kubernetes")
 	startCmd.Flags().StringVar(&startCmdArgs.Kubernetes.Version, "kubernetes-version", defaultKubernetesVersion, "must match a k3s version https://github.com/k3s-io/k3s/releases")
-	startCmd.Flags().BoolVar(&startCmdArgs.Kubernetes.Ingress, "kubernetes-ingress", false, "enable Traefik ingress controller")
+	startCmd.Flags().StringSliceVar(&startCmdArgs.Kubernetes.Disable, "kubernetes-disable", defaultKubernetesDisable, "components to disable for k3s e.g. traefik,servicelb")
 	startCmd.Flag("with-kubernetes").Hidden = true
 
 	// layer
@@ -154,7 +170,25 @@ func init() {
 	startCmd.Flags().StringToStringVar(&startCmdArgs.Env, "env", nil, "environment variables for the VM")
 
 	// dns
-	startCmd.Flags().IPSliceVarP(&startCmdArgs.Network.DNS, "dns", "n", nil, "DNS servers for the VM")
+	startCmd.Flags().IPSliceVarP(&startCmdArgs.Network.DNSResolvers, "dns", "n", nil, "DNS resolvers for the VM")
+	startCmd.Flags().StringSliceVar(&startCmdArgs.Flags.DNSHosts, "dns-host", nil, "Custom DNS names to provide to resolver")
+}
+
+func dnsHostsFromFlag(hosts []string) map[string]string {
+	mapping := make(map[string]string)
+
+	for _, h := range hosts {
+		str := strings.SplitN(h, "=", 2)
+		if len(str) != 2 {
+			log.Warnf("unable to parse custom dns host: %v, skipping\n", h)
+			continue
+		}
+		src := str[0]
+		target := str[1]
+
+		mapping[src] = target
+	}
+	return mapping
 }
 
 // mountsFromFlag converts mounts from cli flag format to config file format
@@ -196,8 +230,19 @@ func prepareConfig(cmd *cobra.Command) {
 
 	// convert cli to config file format
 	startCmdArgs.Mounts = mountsFromFlag(startCmdArgs.Flags.Mounts)
+	startCmdArgs.Network.DNSHosts = dnsHostsFromFlag(startCmdArgs.Flags.DNSHosts)
 	startCmdArgs.ActivateRuntime = &startCmdArgs.Flags.ActivateRuntime
 
+	// handle macOS virtualization.framework transition
+	{
+		if current.VMType == "" {
+			current.VMType = "qemu"
+		}
+		// convert mount type for qemu
+		if startCmdArgs.VMType != "vz" && startCmdArgs.MountType == "virtiofs" {
+			startCmdArgs.MountType = "9p"
+		}
+	}
 	// if there is no existing settings
 	if current.Empty() {
 		// attempt template
@@ -225,6 +270,12 @@ func prepareConfig(cmd *cobra.Command) {
 	if !cmd.Flag("kubernetes").Changed {
 		startCmdArgs.Kubernetes.Enabled = current.Kubernetes.Enabled
 	}
+	if !cmd.Flag("kubernetes-version").Changed {
+		startCmdArgs.Kubernetes.Version = current.Kubernetes.Version
+	}
+	if !cmd.Flag("kubernetes-disable").Changed {
+		startCmdArgs.Kubernetes.Disable = current.Kubernetes.Disable
+	}
 	if !cmd.Flag("runtime").Changed {
 		startCmdArgs.Runtime = current.Runtime
 	}
@@ -246,8 +297,14 @@ func prepareConfig(cmd *cobra.Command) {
 	if !cmd.Flag("ssh-agent").Changed {
 		startCmdArgs.ForwardAgent = current.ForwardAgent
 	}
+	if !cmd.Flag("ssh-config").Changed {
+		startCmdArgs.SSHConfig = current.SSHConfig
+	}
 	if !cmd.Flag("dns").Changed {
-		startCmdArgs.Network.DNS = current.Network.DNS
+		startCmdArgs.Network.DNSResolvers = current.Network.DNSResolvers
+	}
+	if !cmd.Flag("dns-host").Changed {
+		startCmdArgs.Network.DNSHosts = current.Network.DNSHosts
 	}
 	if !cmd.Flag("env").Changed {
 		startCmdArgs.Env = current.Env
@@ -261,12 +318,14 @@ func prepareConfig(cmd *cobra.Command) {
 		}
 	}
 	if util.MacOS() {
-		if !cmd.Flag("network-driver").Changed {
-			startCmdArgs.Network.Driver = current.Network.Driver
-		}
 		if !cmd.Flag("network-address").Changed {
 			startCmdArgs.Network.Address = current.Network.Address
 		}
+		if util.MacOS13OrNewer() {
+			if !cmd.Flag("vm-type").Changed {
+				startCmdArgs.VMType = current.VMType
+			}
+		}
 	}
 }
 
